--[[
	Todo:
		- Add offset option
		- Make multiple markers look nice near the end of the bar and by each other (like this: Player1 (52) >|  |< Player2 (25) )
		- Add an option to show a tooltip when hovering over a marker
		
	Changelog:
		- The levelalways option now works properly
		
		if (strWidth > (barWidth - strPos)) then
			showFlipped()
		end
		
		name: WatchXP-x.x
		filename: WatchXP-x.x.zip
		
]]--

WXP_Version = "1.2.1"
WXP_maxLevel = 80

function WXP_OnLoad()
	WXP_Msg("WatchXP Loaded")
	SlashCmdList["WXP"] = WXP_Slash;
	SLASH_WXP1 = "/watchxp";
	SLASH_WXP2 = "/wxp";
	this:RegisterEvent("PLAYER_XP_UPDATE")
	this:RegisterEvent("CHAT_MSG_ADDON")
	this:RegisterEvent("ADDON_LOADED")
	this:RegisterEvent("PARTY_MEMBERS_CHANGED")
	WXP_TextString = "|< "
	WXP_i = 1
end

function WXP_OnEvent(event, arg1)
	if (event == "PLAYER_XP_UPDATE") then
		WXP_ExpUpdate()
	end
	
	if (event == "ADDON_LOADED") then
		if (arg1 == "WatchXP") then
			WXP_EnumParty()
			SendAddonMessage("WXP","REQ","PARTY")
			if (WXP_Settings == nil) then
				WXP_Settings = {
					["Size"] = 8,
					["Show"] = 1,
					["Debug"] = 0,
					["LevelAlways"] = 0,
					["ClassColor"] = 1
				}
			end
			
			if (WXP_Colors == nil) then
				WXP_Colors = {
					["R"] = 1,
					["G"] = 1,
					["B"] = 1,
					["A"] = 1
				}
			end
			
			--[[if (WXP_Old2 == nil) then
				WXP_Old2 = {
					["otherXP"] = {},
					["otherMax"] = {},
					["Level"] = {},
					["Name"] = {}}
			end]]--
			
			if (WXP_Font == nil) then
				WXP_Font = {
					["Face"] = 1,
					["Size"] = 12}
			end
			
			if (WXP_Settings["Show"] == 0) then
				WXP_Frame:Hide()
			end
			
			if string.match(WXP_Font["Size"],".*(%a).*") then
				WXP_Font["Size"] = 12
			end
			
			if (WXP_Font["Size"] == nil) then
				WXP_Font["Size"] = 12
			end
			
			for i = 1,4 do
				local regions = { WXP_Frame:GetRegions() }
				for _,region in pairs(regions) do
					if (region:IsObjectType("FontString")) then
						if (WXP_Font["Face"] == 1) then
							region:SetFont("Fonts\\FRIZQT__.TTF",WXP_Font["Size"])
						elseif (WXP_Font["Face"] == 2) then
							region:SetFont("Fonts\\ARIALN.TTF",WXP_Font["Size"])
						elseif (WXP_Font["Face"] == 3) then
							region:SetFont("Fonts\\MORPHEUS.TTF",WXP_Font["Size"])
						elseif (WXP_Font["Face"] == 4) then
							region:SetFont("Fonts\\SKURRI.TTF",WXP_Font["Size"])
						else
							region:SetFont("Fonts\\FRIZQT__.TTF",WXP_Font["Size"])
						end
					end
				end
			end
		end
	end
	
	if (event == "CHAT_MSG_ADDON") then
		if (arg1 == "WXP") then
			local msgArgs = {strsplit(",", arg2)}
			if (msgArgs[1] == "NEWXP") then
				if (msgArgs[4] ~= UnitName("player")) then
					WXP_Draw(msgArgs[2],msgArgs[3],msgArgs[4],msgArgs[5])
					WXP_Msg("Sending Draw request",1)
					WXP_Msg("UnitXP: "..msgArgs[2],1)
					WXP_Msg("UnitXPMax: "..msgArgs[3],1)
					WXP_Msg("Name: "..msgArgs[4],1)
					WXP_Msg("Level: "..msgArgs[5],1)
				end
			end
			
			if (msgArgs[1] == "REQ") then
				WXP_ExpUpdate()
				WXP_Msg("Got REQ, running ExpUpdate",1)
			end
			
			if (msgArgs[1] == "PING") then
				SendAddonMessage("WXP","PONG,"..UnitName("player")..","..UnitXP("player")..","..UnitXPMax("player")..","..UnitLevel("player"),"WHISPER",msgArgs[2])
			end
			
			if (msgArgs[1] == "PONG") then
				WXP_Create(msgArgs[2],msgArgs[3],msgArgs[4],msgArgs[5])
			end
		end
	end
	
	if (event == "PARTY_MEMBERS_CHANGED") then
		local regions = { WXP_Frame:GetRegions() }
		for _, region in pairs(regions) do
			if(region:IsObjectType("FontString")) then
				region:SetText("")
			end
		end
		SendAddonMessage("WXP","REQ","PARTY")
		WXP_Msg("Party changed, sending REQ",1)
	end
end

function WXP_ExpUpdate()
	SendAddonMessage("WXP","NEWXP,"..UnitXP("player")..","..UnitXPMax("player")..","..UnitName("player")..","..UnitLevel("player"),"PARTY")
	WXP_Msg("Sending Experience data to party",1)
end

function WXP_Draw(WXP_otherXP,WXP_otherMax,WXP_Name,WXP_Level)
	if (WXP_Level == nil) then
		WXP_Level = "?"
	end
	
	if (WXP_Name == nil) then
		WXP_Name = "Unknown"
	end
	
	if not (WXP_Level == WXP_maxLevel) then
		if (WXP_Name ~= UnitName("player")) then
			if ("WXP_Text_"..WXP_Name) then
				if (WXP_Settings["LevelAlways"] == 1) then
					_G["WXP_Text_"..WXP_Name]:SetText(WXP_TextString..WXP_Name.."("..WXP_Level..")")
				else
					if (tonumber(UnitLevel("player")) == tonumber(WXP_Level)) then
						_G["WXP_Text_"..WXP_Name]:SetText(WXP_TextString..WXP_Name)
					else
						if (_G["WXP_Text_"..WXP_Name]) then
							_G["WXP_Text_"..WXP_Name]:SetText(WXP_TextString..WXP_Name.."("..WXP_Level..")")
						else
							WXP_Create(WXP_Name,WXP_otherXP,WXP_otherMax,WXP_Level)
						end
					end
				end
				_G["WXP_Text_"..WXP_Name]:SetTextColor(WXP_Colors["R"],WXP_Colors["G"],WXP_Colors["B"],WXP_Colors["A"])
				WXP_barWidth = MainMenuExpBar:GetWidth()
				if WXP_otherMax == 0 then
					WXP_xpWidth = 0
				else
					WXP_xpWidth = WXP_barWidth * (WXP_otherXP / WXP_otherMax)
				end
				_G["WXP_Text_"..WXP_Name]:SetPoint("TOPLEFT",WXP_Frame,"TOPLEFT",WXP_xpWidth,0)
			end
		end
	end
end

function WXP_Slash(cmd)
	if (cmd == "regnames") then
		WXP_GetRegNames()

	elseif (cmd == "classcolor") then
		if (WXP_Settings["ClassColor"] == 1) then
			WXP_Msg("Using custom colors for player names")
			WXP_Settings["ClassColor"] = 0
		else
			WXP_Msg("Using class colors for player names")
			WXP_Settings["ClassColor"] = 1
		end

	elseif (cmd == "levelalways") then
		if (WXP_Settings["LevelAlways"] == 1) then
			WXP_Msg("Only showing player level on different levels")
			WXP_Settings["LevelAlways"] = 0
		else
			WXP_Msg("Always showing player level")
			WXP_Settings["LevelAlways"] = 1
		end

	elseif (cmd == "debugdisp") then
		WXP_Draw(1,4,"Debug")

	elseif (cmd == "toggle") then
		WXP_FrameToggle()
	
	elseif (cmd == "debug") then
		if (WXP_Settings["Debug"] == 1) then
			WXP_Msg("Hiding debug data")
			WXP_Settings["Debug"] = 0
		else
			WXP_Msg("Showing debug data")
			WXP_Settings["Debug"] = 1
		end
	
	elseif (cmd == "version") then
		WXP_Msg("Version: "..WXP_Version)
	
	elseif (cmd == "update") then
		SendAddonMessage("WXP","REQ","PARTY")
		
	elseif strmatch(cmd,"color") then
		if (cmd == "color") then
			WXP_Msg("Current color settings: |cffff0000"..WXP_Colors["R"].." |cff00ff00"..WXP_Colors["G"].." |cff0000ff"..WXP_Colors["B"].." |r"..WXP_Colors["A"])
		else
			WXP_Colors["R"] = strmatch(cmd,"color (%d.*%d*) %d.*%d* %d.*%d* %d.*%d*")
			WXP_Colors["G"] = strmatch(cmd,"color %d.*%d* (%d.*%d*) %d.*%d* %d.*%d*")
			WXP_Colors["B"] = strmatch(cmd,"color %d.*%d* %d.*%d* (%d.*%d*) %d.*%d*")
			WXP_Colors["A"] = strmatch(cmd,"color %d.*%d* %d.*%d* %d.*%d* (%d.*%d*)")
			for i = 1,GetNumPartyMembers() do
				if (_G["WXP_Text_"..UnitName("party"..i)]) then
					local regions = { WXP_Frame:GetRegions() }
					for _, region in pairs(regions) do
						if(region:IsObjectType("FontString")) then
							region:SetTextColor(WXP_Colors["R"],WXP_Colors["G"],WXP_Colors["B"],WXP_Colors["A"])
						end
					end
				end
			end
		end
	
	elseif strmatch(cmd,"font") then
		WXP_Font["Size"] = strmatch(cmd,"font (%d%d*) [arial]*[skurri]*[morpheus]*[friz]*")
		local face = strmatch(cmd,"%d%d* ([arial]*[skurri]*[morpheus]*[friz]*)")
			if (face == "friz") then
				WXP_Font["Face"] = 1
				WXP_Msg("Font set to Friz Quadrata TT, size "..WXP_Font["Size"])
				local regions = { WXP_Frame:GetRegions() }
				for _, region in pairs(regions) do
					if(region:IsObjectType("FontString")) then
						region:SetFont("Fonts\\FRIZQT__.TTF",WXP_Font["Size"])
					end
				end

			elseif (face == "arial") then
				WXP_Font["Face"] = 2
				WXP_Msg("Font set to Arial Narrow, size "..WXP_Font["Size"])
				local regions = { WXP_Frame:GetRegions() }
				for _, region in pairs(regions) do
					if(region:IsObjectType("FontString")) then
						region:SetFont("Fonts\\ARIALN.TTF",WXP_Font["Size"])
					end
				end

			elseif (face == "morpheus") then
				WXP_Font["Face"] = 3
				WXP_Msg("Font set to Morpheus, size "..WXP_Font["Size"])
				local regions = { WXP_Frame:GetRegions() }
				for _, region in pairs(regions) do
					if(region:IsObjectType("FontString")) then
						region:SetFont("Fonts\\MORPHEUS.TTF",WXP_Font["Size"])
					end
				end

			elseif (face == "skurri") then
				WXP_Font["Face"] = 4
				WXP_Msg("Font set to Skurri, size "..WXP_Font["Size"])
				local regions = { WXP_Frame:GetRegions() }
				for _, region in pairs(regions) do
					if(region:IsObjectType("FontString")) then
						region:SetFont("Fonts\\SKURRI.TTF",WXP_Font["Size"])
					end
				end

			else
				WXP_Msg("Invalid font face; options are: friz, arial, morpheus, skurri")
			end
	
	else
		WXP_Msg("/wxp toggle : Toggle displaying of WatchXP")
		WXP_Msg("/wxp debug : Toggle displaying of debug information")
		WXP_Msg("/wxp levelalways : Toggle displaying of other player's level when you are the same level")
		WXP_Msg("/wxp version : Display addon version")
		WXP_Msg("/wxp update : Force an update from all party members")
		WXP_Msg("/wxp debugdisp : Draw a placeholder marker on the experience bar for debugging")
		WXP_Msg("/wxp classcolor : Toggle between coloring names by custom color or by class")
		WXP_Msg("/wxp color <Red> <Green> <Blue> <Alpha>: Set font color (i.e. /wxp color 0 0.5 0 1 makes it dark green). Leave blank to see current settings")
		WXP_Msg("/wxp font <size> <face> : Set font style (default size is 12, faces are 'friz' (default), 'arial', 'morpheus', and 'skurri'")
	end
end

function WXP_FrameToggle()
	if (WXP_Settings["Show"] == 1) then
		WXP_Msg("Hiding WatchXP")
		WXP_Frame:Hide()
		WXP_Settings["Show"] = 0
	else
		WXP_Msg("Showing WatchXP")
		WXP_Frame:Show()
		WXP_Settings["Show"] = 1
	end
end

function WXP_Msg(msg,deb)
	if (deb) and (WXP_Settings["Debug"] == 1) then
		DEFAULT_CHAT_FRAME:AddMessage("|cffd2b48c[WatchXP] [DEBUG]|r "..msg)
	else
		if (deb ~= 1) then
			DEFAULT_CHAT_FRAME:AddMessage("|cffd2b48c[WatchXP]|r "..msg)
		end
	end
end

function WXP_Create(WXP_playerName,WXP_otherXP,WXP_otherMax,WXP_Level)
	local newname = "WXP_Text"..WXP_i
	new = WXP_Frame:CreateFontString("WXP_Text_"..WXP_playerName,"ARTWORK","GameFontNormal")
	new:SetText("|< "..WXP_playerName)
	new:SetPoint("TOPLEFT",WXP_Frame,"TOPLEFT")
	new:SetTextColor(WXP_Colors["R"],WXP_Colors["G"],WXP_Colors["B"],WXP_Colors["A"])
	if (WXP_Font["Face"] == 1) then
		new:SetFont("Fonts\\FRIZQT__.TTF",WXP_Font["Size"])
	elseif (WXP_Font["Face"] == 2) then
		new:SetFont("Fonts\\\ARIALN.TTF",WXP_Font["Size"])
	elseif (WXP_Font["Face"] == 3) then
		new:SetFont("Fonts\\MORPHEUS.TTF",WXP_Font["Size"])
	elseif (WXP_Font["Face"] == 4) then
		new:SetFont("Fonts\\SKURRI.TTF",WXP_Font["Size"])
	else
		new:SetFont("Fonts\\FRIZQT__.TTF",WXP_Font["Size"])
	end
	WXP_Draw(WXP_otherXP,WXP_otherMax,WXP_playerName,WXP_Level)
	WXP_i = WXP_i + 1
end

function WXP_EnumParty()
	for i=1,GetNumPartyMembers() do
		SendAddonMessage("WXP","PING,"..UnitName("player"),"WHISPER",UnitName("party"..i))
	end
end

function WXP_GetRegNames()
	local regions = { WXP_Frame:GetRegions() }
	for _, region in pairs(regions) do
		if(region:IsObjectType("FontString")) then
			WXP_Msg(region:GetName())
		end
	end
end
